@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/procmeter3/RCS/procmeter.c 1.8 1999/12/08 19:40:53 amb Exp $
+  $Header: /home/amb/procmeter3/RCS/procmeter.c 1.9 2000/10/22 14:11:36 amb Exp $
 
-  ProcMeter - A system monitoring program for Linux - Version 3.2.
+  ProcMeter - A system monitoring program for Linux - Version 3.2a.
 
   Main program.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1998,99 Andrew M. Bishop
+  This file Copyright 1998,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -131,7 +131,7 @@ int main(int argc,char **argv)
 
     printf("\nProcMeter Version %s\n%s\n\n",PROCMETER_VERSION,&double_underline[sizeof(double_underline)-18-sizeof(PROCMETER_VERSION)]);
     printf("An efficient modular system monitoring program for Linux.\n");
-    printf("(c) Andrew M. Bishop 1998,99 [amb@gedanken.demon.co.uk]\n\n");
+    printf("(c) Andrew M. Bishop 1998,99,2000 [amb@gedanken.demon.co.uk]\n\n");
 
     printf("Usage: ProcMeter [-h] [--rc=<filename>] [--...] [...]\n\n");
 
